﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using C1.Win.Chart;
using C1.Chart;
using AxisGroupingDemo.Data.Model;

namespace AxisGroupingDemo.Samples
{
    public partial class DateTimeAxisGrouping : UserControl
    {
        private FlexChart flexChart1;
        private Random rnd;
        private DateTimeGroupProvider _groupProvider;
        private CheckBox chkAllowGrouping;
        private CheckBox chkShowYears;
        private CheckBox chkShowQuarters;
        private CheckBox chkShowMonths;

        public DateTimeAxisGrouping()
        {
            InitializeComponent();
            InitializeControls();
            SetupChart();
            chkAllowGrouping.Checked = true;
        }

        private void SetupChart()
        {
            flexChart1.BeginUpdate();

            flexChart1.ChartType = ChartType.Line;
            flexChart1.DataSource = CreateData();
            flexChart1.BindingX = "Time";
            flexChart1.Binding = "Price";
            flexChart1.AxisX.LabelAngle = 90;
            flexChart1.AxisX.MajorUnit = 10;
            flexChart1.AxisX.Format = "dd";
            flexChart1.AxisX.GroupProvider = _groupProvider;
            flexChart1.AxisX.GroupSeparator = AxisGroupSeparator.Grid;
            flexChart1.Series.Clear();
            var series = new Series();
            flexChart1.Series.Add(series);
            flexChart1.EndUpdate();
        }

        private void InitializeControls()
        {
            baseSample1.lblTitle.Text = Properties.Resources.DateTimeAxisTitle;
            baseSample1.tbDescription.Text = Properties.Resources.DateTimeAxisDesc;

            rnd = new Random();
            _groupProvider = new DateTimeGroupProvider();

            flexChart1 = baseSample1.flexChart1;

            chkAllowGrouping = new CheckBox
            {
                Text = "Allow Grouping",
                Checked = true
            };

            chkShowYears = new CheckBox
            {
                Text = "Years",
                Checked = true,
                Tag = DateTimeGroup.Year
            };

            chkShowQuarters = new CheckBox
            {
                Text = "Quarters",
                Checked = true,
                Tag = DateTimeGroup.Quarter
            };

            chkShowMonths = new CheckBox
            {
                Text = "Months",
                Checked = true,
                Tag = DateTimeGroup.Month
            };

            chkShowYears.CheckedChanged += OnShowGroupChanged;
            chkShowQuarters.CheckedChanged += OnShowGroupChanged;
            chkShowMonths.CheckedChanged += OnShowGroupChanged;
            chkAllowGrouping.CheckedChanged += OnAllowGroupingChanged;

            baseSample1.pControls.Controls.Add(chkAllowGrouping);
            baseSample1.pControls.Controls.Add(chkShowYears);
            baseSample1.pControls.Controls.Add(chkShowQuarters);
            baseSample1.pControls.Controls.Add(chkShowMonths);
        }

        private void OnAllowGroupingChanged(object sender, EventArgs e)
        {
            if (chkAllowGrouping.Checked)
            {
                _groupProvider.AllowGrouping = true;
                flexChart1.AxisX.Format = "dd";
                flexChart1.AxisX.LabelAngle = double.NaN;
                flexChart1.Refresh();
                chkShowYears.Visible = chkShowMonths.Visible = chkShowQuarters.Visible = true;
            }
            else
            {
                _groupProvider.AllowGrouping = false;
                flexChart1.AxisX.Format = "dd-MMM-yyyy";
                flexChart1.AxisX.LabelAngle = 90;
                flexChart1.Refresh();
                chkShowYears.Visible = chkShowMonths.Visible = chkShowQuarters.Visible = false;
            }
        }

        private void OnShowGroupChanged(object sender, EventArgs e)
        {
            var chk = sender as CheckBox;
            var showGroup = (DateTimeGroup)chk.Tag;
            if (chk.Checked)
                _groupProvider.ShownGroups = _groupProvider.ShownGroups | showGroup;
            else
                _groupProvider.ShownGroups = _groupProvider.ShownGroups ^ showGroup;
            flexChart1.Refresh();
        }

        List<Quote> CreateData()
        {
            List<Quote> list = new List<Quote>();
            var dt = DateTime.Today;
            var price = 1000;
            for (var i = 0; i < 365; i++)
            {
                price += rnd.Next(10) % 2 == 0 ? rnd.Next(50) : -rnd.Next(50);
                list.Add(new Quote() { Time = dt.AddDays(i), Price = price });
            }

            return list;
        }
    }

    [Flags]
    public enum DateTimeGroup
    {
        Month = 1,
        Quarter = 2,
        Year = 4
    }

    public class DateTimeGroupProvider : IAxisGroupProvider
    {
        public DateTimeGroup ShownGroups { get; set; }

        public bool AllowGrouping { get; set; }

        public DateTimeGroupProvider()
        {
            AllowGrouping = true;
            ShownGroups = DateTimeGroup.Month | DateTimeGroup.Quarter | DateTimeGroup.Year;
        }

        public int GetLevels(IRange range)
        {
            return AllowGrouping ? 3 : 0;
        }

        public IList<IRange> GetRanges(IRange range, int level)
        {
            var timeRange = range as TimeRange;
            if (timeRange == null)
                return null;
            var min = timeRange.TimeMin;
            var max = timeRange.TimeMax;

            var span = max - min;

            List<IRange> ranges = new List<IRange>();
            DateTime start;
            switch (level)
            {
                case 1:
                    if (!ShownGroups.HasFlag(DateTimeGroup.Month))
                        return null;
                    start = new DateTime(min.Year, min.Month, 1);
                    ranges = Enumerable.Range(0, ((max.Year - min.Year) * 12) + max.Month - min.Month + 1).Select(a => start.AddMonths(a))
                   .TakeWhile(a => a <= max)
                   .Select(a => (IRange)(new TimeRange(a.ToString("MMM"), a, a.AddMonths(1)))).ToList();

                    break;
                case 2:
                    if (!ShownGroups.HasFlag(DateTimeGroup.Quarter))
                        return null;
                    start = new DateTime(min.Year, ((int)Math.Ceiling((double)min.Month / 3) - 1) * 3 + 1, 1);
                    ranges = Enumerable.Range(0, ((max.Month - start.Month) / 3 + 1) + 4 * (max.Year - start.Year)).Select(a => start.AddMonths(a * 3))
                   .TakeWhile(a => a <= max)
                   .Select(a => (IRange)(new TimeRange("Q" + (int)Math.Ceiling((double)a.Month / 3), a, a.AddMonths(3)))).ToList();
                    break;
                case 3:
                    if (!ShownGroups.HasFlag(DateTimeGroup.Year))
                        return null;
                    start = new DateTime(min.Year, 1, 1);
                    ranges = Enumerable.Range(0, max.Year - start.Year + 1).Select(a => start.AddYears(a))
                   .TakeWhile(a => a <= max)
                   .Select(a => (IRange)(new TimeRange(a.ToString("yyyy"), a, a.AddYears(1)))).ToList();
                    break;
            }
            return ranges;
        }
    }
}
